@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 ### BEGIN LICENSE
-# Copyright (C) 2010-2011 Emilien Klein <emilien _AT_ klein _DOT_ st>
+# Copyright (C) 2010-2012 Emilien Klein <emilien _AT_ klein _DOT_ st>
 # 
 # This program is free software: you can redistribute it and/or modify it 
 # under the terms of the GNU General Public License version 3, as published 
@@ -100,8 +100,8 @@ class NautilusImageManipulatorDialog(Gtk.Dialog):
                 # If the appendString ends in "/", the image would be
                 # called ".EXT", which is a hidden file in it's own folder.
                 self.error_with_parameters(
-                       _("The string to append cannot end in %s" %
-                       os.path.sep))
+                       _("The string to append cannot end in %s") %
+                       os.path.sep)
                 return
             # TODO: Check that the value is valid to be appended to the filename
 
@@ -111,23 +111,25 @@ class NautilusImageManipulatorDialog(Gtk.Dialog):
         
         # Determine the resizing parameters
         geometry = None # The size parameters for the resizing operation
+        aspect = False # Custom aspect ratio disabled
         # Resize using default values
         if self.builder.get_object("default_size_radiobutton").get_active():
             model = self.builder.get_object("size_combobox").get_model()
             iterator = self.builder.get_object("size_combobox").get_active_iter()
             geometry = model.get_value(iterator, 0)
 
-
-
-
         # Resize using a custom scale value
         elif self.builder.get_object("custom_scale_radiobutton").get_active():
             geometry = "%d%%" % int(self.builder.get_object("scale_spinbutton").get_value())
 
         # Resize using custom scale values
         elif self.builder.get_object("custom_size_radiobutton").get_active():
+            aspect = self.builder.get_object("aspect_checkbutton").get_active()
             geometry = "%dx%d" % (int(self.builder.get_object("width_spinbutton").get_value()), int(self.builder.get_object("height_spinbutton").get_value()))
         
+        # Compression level
+        compression = "%d" % (int(self.builder.get_object("compression_spinbutton").get_value()))
+        
         if geometry:
             # Disable the parameter UI elements and display the progress bar
             self.builder.get_object("parameters_vbox").set_sensitive(False)
@@ -137,11 +139,11 @@ class NautilusImageManipulatorDialog(Gtk.Dialog):
             while Gtk.events_pending():
                 Gtk.main_iteration() # Used to refresh the UI
             # Resize the images
-            im = ImageManipulations(self, self.files, geometry, subdirectoryName, appendString)
+            im = ImageManipulations(self, self.files, geometry, aspect, compression, subdirectoryName, appendString)
             im.connect("resizing_done", self.on_resizing_done)
             task = im.resize_images()
             GObject.idle_add(task.next)
-
+        
         # Remember the settings for next time
         self.saveConfig()
 
@@ -296,9 +298,23 @@ class NautilusImageManipulatorDialog(Gtk.Dialog):
             isCustomSize = (widget == self.builder.get_object("custom_size_radiobutton"))
             self.builder.get_object("custom_width_label").set_sensitive(isCustomSize)
             self.builder.get_object("width_spinbutton").set_sensitive(isCustomSize)
-            self.builder.get_object("custom_height_label").set_sensitive(isCustomSize)
-            self.builder.get_object("height_spinbutton").set_sensitive(isCustomSize)
+            self.builder.get_object("aspect_frame").set_sensitive(isCustomSize)
+            is_aspect_checkbutton_toggled = self.builder.get_object(
+                                        "aspect_checkbutton").get_active()
+            self.builder.get_object("custom_height_label").set_sensitive(
+                    isCustomSize and is_aspect_checkbutton_toggled)
+            self.builder.get_object("height_spinbutton").set_sensitive(
+                    isCustomSize and is_aspect_checkbutton_toggled)
             self.builder.get_object("custom_pixels_label").set_sensitive(isCustomSize)
+            
+    def on_aspect_toggled(self, widget, data=None):
+        """Updates the sensitiveness of the elements involved with aspect ratio."""
+        is_aspect_checkbutton_toggled = self.builder.get_object(
+                                        "aspect_checkbutton").get_active()
+        self.builder.get_object("custom_height_label").set_sensitive(
+                                             is_aspect_checkbutton_toggled)
+        self.builder.get_object("height_spinbutton").set_sensitive(
+                                             is_aspect_checkbutton_toggled)
 
     def on_filename_toggled(self, widget, data=None):
         """Updates the sensitiveness of the filename entry boxes depending on which option is chosen."""
@@ -331,19 +347,15 @@ class NautilusImageManipulatorDialog(Gtk.Dialog):
         response = dialog.run()
         dialog.destroy()
 
-    def error_resizing(self, filename=None, dependencyMissing=False):
-        """Displays an error message if ImageMagick returned an error while resizing one image."""
+    def error_resizing(self, filename):
+        """Displays an error message if an error got detected while
+        resizing one image."""
         label = None
-        if filename:
-            (folder, image) = os.path.split(filename)
-            buttons =(_("_Skip"), 0,
-                   Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
-                   _("_Retry"), 1)
-            label = Gtk.Label(label=_('The image "%(image)s" could not be resized.\n\nCheck whether you have permission to write to this folder:\n%(folder)s' % {"image": image, "folder": folder}))
-        if dependencyMissing:
-            buttons =(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
-                   _("_Retry"), 1)
-            label = Gtk.Label(label=_("The image conversion program wasn't found on your system.\nHave you installed ImageMagick?"))
+        (folder, image) = os.path.split(filename)
+        buttons =(_("_Skip"), 0,
+               Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
+               _("_Retry"), 1)
+        label = Gtk.Label(label=_('The image "%(image)s" could not be resized.\n\nCheck whether you have permission to write to this folder:\n%(folder)s' % {"image": image, "folder": folder}))
         label.set_padding(10, 5)
         dialog = Gtk.Dialog(_("Could not resize image"),
                            self,
@@ -359,6 +371,19 @@ class NautilusImageManipulatorDialog(Gtk.Dialog):
         retry = (response == 1)
         return (skip, self.processingCanceled, retry)
 
+    def readConfigValue(self, section, name, defaultValue=None, type=None):
+        value = defaultValue
+        try:
+            if type == "int":
+                value = self.config.getint(section, name)
+            elif type == "boolean":
+                value = self.config.getboolean(section, name)
+            else:
+                value = self.config.get(section, name)
+        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
+            pass
+        return value
+
     def loadConfig(self):
         """Read the ini file to get the previous configuration. The ini file is located at ~/.nautilus-image-manipulator.ini.
         
@@ -366,46 +391,50 @@ class NautilusImageManipulatorDialog(Gtk.Dialog):
         self.configFilename = os.path.expanduser("~/.nautilus-image-manipulator.ini")
         self.config = ConfigParser.ConfigParser()
         self.config.read(self.configFilename)
-        if len(self.config.sections()) > 0:
-            # TODO: take care of eventual exceptions if a value is not set
-            # Resize
-            size_combobox_value = self.config.getint("Resize", "size_combobox")
-            scale_adjustment_value = self.config.getint("Resize", "scale_adjustment")
-            width_adjustment_value = self.config.getint("Resize", "width_adjustment")
-            height_adjustment_value = self.config.getint("Resize", "height_adjustment")
-            toggled_size_radiobutton = self.config.get("Resize", "toggled_size_radiobutton")
-            
-            # Output
-            subdirectory_name_entry_value = self.config.get("Output", "subdirectory_name_entry")
-            append_name_entry_value = self.config.get("Output", "append_name_entry")
-            toggled_output_radiobutton = self.config.get("Output", "toggled_output_radiobutton")
-            
-            # Sending
-            is_send_checkbutton_toggled = self.config.getboolean("Sending", "is_send_checkbutton_toggled")
-            toggled_sending_option_radiobutton = self.config.get("Sending", "toggled_sending_option_radiobutton")
-            upload_combobox_value = self.config.getint("Sending", "upload_combobox")
-            
-            # TODO: Make sure that the values read from the ini file are valid, else use default values
-        else:
-            # Default parameters
-            # Resize
-            size_combobox_value = 4
-            scale_adjustment_value = 50
-            width_adjustment_value = 1000
-            height_adjustment_value = 1000
-            toggled_size_radiobutton = "default_size_radiobutton"
-            
-            # Output
-            toggled_output_radiobutton = "subdirectory_radiobutton"
-            # Default name of the subdirectory in which the resized images will be put
-            subdirectory_name_entry_value = _("resized")
-            # Default value of the string that will be appended to the filename of the resized images
-            append_name_entry_value = _("-resized")
-            
-            # Sending
-            is_send_checkbutton_toggled = False
-            toggled_sending_option_radiobutton = "upload_radiobutton"
-            upload_combobox_value = 0
+        
+        # TODO: Make sure that the values read from the ini file are valid, else use default values
+        
+        # Resize
+        size_combobox_value = self.readConfigValue("Resize",
+                                                   "size_combobox",4, "int")
+        scale_adjustment_value = self.readConfigValue("Resize",
+                                                      "scale_adjustment",
+                                                      50, "int")
+        width_adjustment_value = self.readConfigValue("Resize",
+                                                      "width_adjustment",
+                                                      1000, "int")
+        height_adjustment_value = self.readConfigValue("Resize",
+                                                       "height_adjustment",
+                                                       1000, "int")
+        is_aspect_checkbutton_toggled = self.readConfigValue("Resize",
+                                           "is_aspect_checkbutton_toggled",
+                                           False, "boolean")
+        compression_adjustement_value = self.readConfigValue("Resize",
+                                                "compression_adjustment",
+                                                95, "int")
+        toggled_size_radiobutton = self.readConfigValue("Resize",
+                                                "toggled_size_radiobutton",
+                                                "default_size_radiobutton")
+        
+        # Output
+        toggled_output_radiobutton = self.readConfigValue("Output",
+                                              "toggled_output_radiobutton",
+                                              "subdirectory_radiobutton")
+        # Default name of the subdirectory in which the resized images will be put
+        subdirectory_name_entry_value = self.readConfigValue("Output", "subdirectory_name_entry", _("resized"))
+        # Default value of the string that will be appended to the filename of the resized images
+        append_name_entry_value = self.readConfigValue("Output", "append_name_entry", _("-resized"))
+        
+        # Sending
+        is_send_checkbutton_toggled = self.readConfigValue("Sending",
+                                           "is_send_checkbutton_toggled",
+                                           False, "boolean")
+        toggled_sending_option_radiobutton = self.readConfigValue("Sending",
+                                       "toggled_sending_option_radiobutton",
+                                       "upload_radiobutton")
+        upload_combobox_value = self.readConfigValue("Sending",
+                                                     "upload_combobox",
+                                                     0, "int")
         
         # Update the UI with the previous (or default) values
         # Size parameters
@@ -413,6 +442,8 @@ class NautilusImageManipulatorDialog(Gtk.Dialog):
         self.builder.get_object("scale_adjustment").set_value(scale_adjustment_value)
         self.builder.get_object("width_adjustment").set_value(width_adjustment_value)
         self.builder.get_object("height_adjustment").set_value(height_adjustment_value)
+        self.builder.get_object("aspect_checkbutton").set_active(is_aspect_checkbutton_toggled)
+        self.builder.get_object("compression_adjustment").set_value(compression_adjustement_value)
         self.builder.get_object(toggled_size_radiobutton).set_active(True)
         
         # Output parameters
@@ -426,14 +457,15 @@ class NautilusImageManipulatorDialog(Gtk.Dialog):
         self.builder.get_object("upload_combobox").set_active(upload_combobox_value)
 
     def saveConfig(self):
-        """Save the current configuration in the ini file"""
+        """Save the current configuration to the ini file"""
         f = open(self.configFilename, "w")
         
         if not self.config.has_section("Resize"):
             self.config.add_section("Resize")
         self.config.set("Resize", "size_combobox", int(self.builder.get_object("size_combobox").get_active()))
-        for v in ("scale_adjustment", "width_adjustment", "height_adjustment"):
+        for v in ("scale_adjustment", "width_adjustment", "height_adjustment", "compression_adjustment"):
             self.config.set("Resize", v, int(self.builder.get_object(v).get_value()))
+        self.config.set("Resize", "is_aspect_checkbutton_toggled", self.builder.get_object("aspect_checkbutton").get_active())
         for b in ("default_size_radiobutton", "custom_scale_radiobutton", "custom_size_radiobutton"):
             if self.builder.get_object(b).get_active():
                 self.config.set("Resize", "toggled_size_radiobutton", b)
